@@ -48,21 +48,33 @@ pub enum Strategy {
     S10,
 }
 
-setting! {
-    /// Differential Evolution settings.
-    pub struct De {
-        @base,
-        @pop_num = 400,
-        /// Strategy of the formula.
-        strategy: Strategy = S1,
-        /// F factor.
-        f: f64 = 0.6,
-        /// Crossing probability.
-        cross: f64 = 0.9,
+/// Differential Evolution settings.
+pub struct De {
+    /// Base setting.
+    pub base: BasicSetting,
+    /// Strategy of the formula.
+    pub strategy: Strategy,
+    /// F factor.
+    pub f: f64,
+    /// Crossing probability.
+    pub cross: f64,
+}
+
+impl Default for De {
+    fn default() -> Self {
+        Self {
+            base: BasicSetting {
+                pop_num: 400,
+                ..Default::default()
+            },
+            strategy: S1,
+            f: 0.6,
+            cross: 0.9,
+        }
     }
 }
 
-impl Setting for De {
+impl<F: ObjFunc> Setting<F> for De {
     type Algorithm = Method;
 
     fn base(&self) -> &BasicSetting {
@@ -124,9 +136,9 @@ impl Method {
         v: Array1<usize>,
         mut n: usize,
     ) {
-        for _ in 0..ctx.dim {
+        for _ in 0..ctx.dim() {
             tmp[n] = self.formula(ctx, tmp, &v, n);
-            n = (n + 1) % ctx.dim;
+            n = (n + 1) % ctx.dim();
             if !maybe(self.cross) {
                 break;
             }
@@ -140,35 +152,35 @@ impl Method {
         v: Array1<usize>,
         mut n: usize,
     ) {
-        for lv in 0..ctx.dim {
-            if !maybe(self.cross) || lv == ctx.dim - 1 {
+        for lv in 0..ctx.dim() {
+            if !maybe(self.cross) || lv == ctx.dim() - 1 {
                 tmp[n] = self.formula(ctx, tmp, &v, n);
             }
-            n = (n + 1) % ctx.dim;
+            n = (n + 1) % ctx.dim();
         }
     }
 }
 
-impl Algorithm for Method {
-    fn generation<F: ObjFunc>(&mut self, ctx: &mut Context<F>) {
-        'a: for i in 0..ctx.pop_num {
+impl<F: ObjFunc> Algorithm<F> for Method {
+    fn generation(&mut self, ctx: &mut Context<F>) {
+        'a: for i in 0..ctx.pop_num() {
             // Generate Vector
             let mut v = Array1::zeros(self.num);
-            rand_vector(v.as_slice_mut().unwrap(), 0, 0, ctx.pop_num);
+            rand_vector(v.as_slice_mut().unwrap(), 0, 0, ctx.pop_num());
             // Recombination
             let mut tmp = ctx.pool.slice(s![i, ..]).to_owned();
-            let n = rand_int(0, ctx.dim);
+            let n = rand_int(0, ctx.dim());
             match self.strategy {
                 S1 | S2 | S3 | S4 | S5 => self.c1(ctx, &mut tmp, v, n),
                 S6 | S7 | S8 | S9 | S10 => self.c2(ctx, &mut tmp, v, n),
             }
-            for s in 0..ctx.dim {
+            for s in 0..ctx.dim() {
                 if tmp[s] > ctx.ub(s) || tmp[s] < ctx.lb(s) {
                     continue 'a;
                 }
             }
             let tmp_f = ctx.func.fitness(tmp.as_slice().unwrap(), &ctx.report);
-            if tmp_f < ctx.fitness[i] {
+            if tmp_f.value() < ctx.fitness[i].value() {
                 ctx.assign_from(i, tmp_f, &tmp);
             }
         }
