@@ -1,17 +1,20 @@
-#[cfg(feature = "std")]
-extern crate std;
-#[cfg(feature = "std")]
-use std::time::Instant;
-
 /// The data of generation sampling.
 #[derive(Clone, Debug)]
 pub struct Report {
     /// Generation.
     pub gen: u32,
-    /// The best fitness.
+    /// Best fitness.
     pub best_f: f64,
-    /// The gradient of the best fitness, between the current and the previous.
+    /// Is the best fitness feasible.
+    pub best_feasible: bool,
+    /// Gradient of the best fitness, between the current and the previous.
     pub diff: f64,
+    /// Average of the finite-fitness individuals.
+    ///
+    /// The first value might be [`f64::NAN`].
+    pub average: f64,
+    /// Adaptive factor.
+    pub adaptive: f64,
     /// Time duration.
     #[cfg(feature = "std")]
     #[cfg_attr(doc_cfg, doc(cfg(feature = "std")))]
@@ -23,24 +26,12 @@ impl Default for Report {
         Self {
             gen: 0,
             best_f: f64::INFINITY,
+            best_feasible: false,
             diff: 0.,
+            average: f64::NAN,
+            adaptive: 0.,
             #[cfg(feature = "std")]
             time: 0.,
         }
     }
 }
-
-impl Report {
-    pub(crate) fn next_gen(&mut self) {
-        self.gen += 1;
-    }
-
-    pub(crate) fn set_diff(&mut self, diff: f64) {
-        self.diff = diff;
-    }
-
-    #[cfg(feature = "std")]
-    pub(crate) fn update_time(&mut self, time: Instant) {
-        self.time = (Instant::now() - time).as_secs_f64();
-    }
-}
