@@ -12,12 +12,13 @@ impl Default for TestObj {
 
 impl ObjFunc for TestObj {
     type Result = f64;
+    type Respond = f64;
 
-    fn fitness(&self, v: &[f64], _: &Report) -> f64 {
+    fn fitness(&self, v: &[f64], _: &Report) -> Self::Respond {
         OFFSET + v[0] * v[0] + 8. * v[1] * v[1] + v[2] * v[2] + v[3] * v[3]
     }
 
-    fn result(&self, v: &[f64]) -> f64 {
+    fn result(&self, v: &[f64]) -> Self::Result {
         self.fitness(v, &Default::default())
     }
 
@@ -29,12 +30,13 @@ impl ObjFunc for TestObj {
     }
 }
 
-fn test<S: Setting>(obj: TestObj, setting: S) {
-    let a = Solver::solve(obj, setting, |_| true);
+fn test(setting: impl Setting<TestObj>) {
+    let a = Solver::solve(TestObj::default(), setting, |_| true);
     let ans = a.result();
-    let (x, y) = a.parameters();
-    let history = a.history();
-    assert!(history.len() > 0, "{}", history.len());
+    let x = a.best_parameters();
+    let y = a.best_fitness();
+    let reports = a.reports();
+    assert!(reports.len() > 0, "{}", reports.len());
     assert!((ans - OFFSET).abs() < 1e-20, "{}", ans);
     for i in 0..4 {
         assert!(x[i].abs() < 1e-6, "x{} = {}", i, x[i]);
@@ -44,34 +46,33 @@ fn test<S: Setting>(obj: TestObj, setting: S) {
 
 #[test]
 fn de() {
-    test(TestObj::default(), De::default().task(Task::MinFit(OFFSET)));
+    test(setting!(De {
+        +base: { task: Task::MinFit(OFFSET) }
+    }));
 }
 
 #[test]
 fn pso() {
-    test(
-        TestObj::default(),
-        Pso::default().task(Task::MinFit(OFFSET)),
-    );
+    test(setting!(Pso {
+        +base: { task: Task::MinFit(OFFSET) }
+    }));
 }
 
 #[test]
 fn fa() {
-    test(TestObj::default(), Fa::default().task(Task::MinFit(OFFSET)));
+    test(setting!(Fa {
+        +base: { task: Task::MinFit(OFFSET) }
+    }));
 }
 
 #[test]
 fn rga() {
-    test(
-        TestObj::default(),
-        Rga::default().task(Task::MinFit(OFFSET)),
-    );
+    test(setting!(Rga {
+        +base: { task: Task::MinFit(OFFSET) }
+    }));
 }
 
 #[test]
 fn tlbo() {
-    test(
-        TestObj::default(),
-        Tlbo::default().task(Task::MinFit(OFFSET)),
-    );
+    test(setting!(Tlbo(task: Task::MinFit(OFFSET))));
 }
